"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBootCommand = exports.wrapExecCommand = exports.wrapTryCatch = exports.readLines = exports.splitJobsByFound = exports.throwYellow = exports.logBlue = exports.logYellow = exports.logGreen = exports.answer = exports.msLink = exports.jqLink = exports.logArray = exports.getTimeAgoFilter = exports.showJobs = exports.getJob = exports.LAST_SAVED_CONNECTION_NAME = void 0;
const fs_1 = __importDefault(require("fs"));
const readline_1 = __importDefault(require("readline"));
const node_jq_1 = require("node-jq");
const chalk_1 = __importDefault(require("chalk"));
const ms_1 = __importDefault(require("ms"));
const queue_1 = require("./queue");
exports.LAST_SAVED_CONNECTION_NAME = "_last-active";
const getJob = async (jobId) => {
    const queue = await (0, queue_1.getQueue)();
    const job = await queue.getJob(jobId);
    if (!job) {
        return (0, exports.throwYellow)(`Job "${jobId}" not found`);
    }
    return job;
};
exports.getJob = getJob;
const showJobs = async (arr, query) => {
    var _a;
    const jobs = arr;
    const root = jobs
        .filter(j => j)
        .map(job => ({
        id: job.id,
        data: job.data,
        timestamp: Number.isNaN(job.timestamp)
            ? job.timestamp
            : new Date(job.timestamp),
        processedOn: job.processedOn && new Date(job.processedOn),
        finishedOn: job.finishedOn && new Date(job.finishedOn),
        name: job.name,
        failedReason: job.failedReason,
        stackTrace: job.stacktrace,
        returnValue: job.returnvalue,
        attemptsMade: job.attemptsMade,
        delay: job.delay,
        progress: job.progress
    }));
    const filteredData = query ? (await (0, node_jq_1.run)(query, { root }, { input: 'json', output: 'json' })) : { root };
    (0, exports.logArray)((_a = filteredData === null || filteredData === void 0 ? void 0 : filteredData.root) !== null && _a !== void 0 ? _a : filteredData);
};
exports.showJobs = showJobs;
const getTimeAgoFilter = (timeAgo) => {
    return new Promise(resolve => {
        try {
            const msAgo = timeAgo && timeAgo.length ? (0, ms_1.default)(timeAgo) : void 0;
            const filter = msAgo ? `{root: [.root[] | select((.timestamp | split(".")[0] | strptime("%Y-%m-%dT%H:%M:%S") | mktime | . * 1000) >= ${Date.now() - msAgo})]}` : '';
            resolve(filter);
        }
        catch (e) {
            (0, exports.throwYellow)(`Error: Argument to --timeAgo is invalid: ${e}`);
        }
    });
};
exports.getTimeAgoFilter = getTimeAgoFilter;
const logArray = (arr) => {
    console.dir(arr, {
        colors: true,
        depth: null,
        maxArrayLength: Infinity,
    });
    Array.isArray(arr) && console.log(`count: ${chalk_1.default.yellow(arr.length)}`);
};
exports.logArray = logArray;
exports.jqLink = "https://stedolan.github.io/jq/manual/#Basicfilters";
exports.msLink = "https://github.com/zeit/ms#examples";
const answer = async (vorpal, question, forceYes) => {
    if (forceYes) {
        (0, exports.logYellow)(`Assuming yes for ${question}`);
        return;
    }
    const answer = (await vorpal.activeCommand.prompt({
        name: "a",
        message: `${question}? (y/n): `
    }));
    if (answer.a !== "y") {
        (0, exports.throwYellow)("You cancel action");
    }
};
exports.answer = answer;
const logGreen = (msg) => {
    console.log(chalk_1.default.green(msg));
};
exports.logGreen = logGreen;
const logYellow = (msg) => {
    console.log(chalk_1.default.yellow(msg));
};
exports.logYellow = logYellow;
const logBlue = (msg) => {
    console.log(chalk_1.default.blueBright(msg));
};
exports.logBlue = logBlue;
const throwYellow = (msg) => {
    let err = new Error();
    err.yellow = true;
    err.stack = chalk_1.default.yellow(msg);
    throw err;
};
exports.throwYellow = throwYellow;
async function splitJobsByFound(jobIds) {
    const queue = await (0, queue_1.getQueue)();
    const jobs = await Promise.all(jobIds.map(id => queue.getJob(id)));
    let notFoundIds = [];
    let foundJobs = [];
    let i = 0;
    for (const jobId of jobIds) {
        const job = jobs[i];
        if (job) {
            foundJobs.push(job);
        }
        else {
            notFoundIds.push(jobId);
        }
        i++;
    }
    return { notFoundIds, foundJobs };
}
exports.splitJobsByFound = splitJobsByFound;
function readLines(file) {
    const fileStream = fs_1.default.createReadStream(file);
    const rl = readline_1.default.createInterface({
        input: fileStream,
        crlfDelay: Infinity
    });
    return rl;
}
exports.readLines = readLines;
function wrapTryCatch(fn) {
    return async function (args) {
        try {
            return await fn.call(this, args);
        }
        catch (e) {
            if (e.yellow) {
                throw e;
            }
            return (0, exports.throwYellow)(e.message);
        }
    };
}
exports.wrapTryCatch = wrapTryCatch;
function wrapExecCommand(args) {
    const index = args.findIndex(item => {
        if (item === '-e' || item === '--exec') {
            return true;
        }
    });
    if (index > 0) {
        args[index + 1] = '"' + args[index + 1] + '"';
    }
}
exports.wrapExecCommand = wrapExecCommand;
function getBootCommand() {
    const args = process.argv.slice(2);
    wrapExecCommand(args);
    return args.join(' ');
}
exports.getBootCommand = getBootCommand;
